/*
 *  AudioEvent.h
 *  EDL
 *
 *  Created by Jerry on Sat Feb 07 2004.
 *  Copyright (c) 2004 Frontier Design Group. All rights reserved.
 *
 */

#ifndef __cAUDIOEVENT_CPP
#define __cAUDIOEVENT_CPP

#include "hdFile.h"
#include "hdLinkedList.h"
#include <string.h>

#define	FBUF_SIZE	    131072              // 128k
#define	FBUF_MIDWAY	     65536              // half buf, 64k

#define	FBUF_SIZE_24    122880              // slighly smaller for 24 bit divisable
#define	FBUF_MIDWAY_24	 61440


// temp repository for data read from files
// 
class fileBuffer 
{
protected:
	bool            splitBuf;			// does it have both left & right data?
	QWORD           startSample;
	QWORD           samples;
	char*           data;
	hdFile*         theFile;

public:
    fileBuffer();
    ~fileBuffer();

public:
	inline bool     InBuffer(QWORD sample)  { return (sample >= startSample && sample < End()); }
	inline QWORD    Samples()               { return samples;               }
	inline QWORD    Start()                 { return startSample;           }
	inline QWORD    End()                   { return startSample + samples; }
	inline char*    Data()                  { return data;                  }
	inline char*    LData()                 { return data + FBUF_MIDWAY;    }

    QWORD           SamplesLeft(QWORD start);
	char*           LeftData(QWORD start );
	char*           RightData(QWORD start );

	inline void     Initialize(bool split, hdFile* file);
	inline void	    Start(QWORD sample) { startSample = sample; }
	inline void	    Samples(QWORD count) { samples = count; }

    inline void     ZeroBuffer()			{ memset(data,0,FBUF_SIZE); }
	DWORD  Max();
    DWORD  MaxBytes();

};



class AudioEvent 
{
public:
	AudioEvent(hdFile* file, hdFile* cfile, ChanTypes);
	AudioEvent(const AudioEvent* event);
	~AudioEvent();
	
private:
	// members
	hdFile*			theFile;			// handle/ref to file related to this event
	hdFile*			compFile;			// for cases of dual mono files (single event)
	ChanTypes		atype;				// type of file
	QWORD			sampleStart;		// start point in project timeline
	QWORD			sampleEnd;			// computed end point in samples
	QWORD			sampleSize;			// number of samples in event
	QWORD			fileSampleOffset;	// offset in samples from start of file
	AudioEvent*		next;
	AudioEvent*		prev;

	fileBuffer	    fbuf;

public:
	inline ChanTypes	Type()			{ return atype;  }
	inline QWORD	Size()				{ return sampleSize;  }
	inline QWORD	End()				{ return sampleEnd;   }
	inline QWORD	Start()				{ return sampleStart; }
	inline hdFile*	File()				{ return theFile;  }
	inline QWORD	FileSampleStart()	{ return fileSampleOffset;  }

	bool			NextHasSameID()		{ return (next && theFile && next->File() && theFile->SameFileID(next->File())); }

	// file accessors
	inline WORD		Width()				{ return (theFile) ? theFile->BitsPerSample() : 0; }
    fileBuffer*     FileBuf()			{ return &fbuf; }

	AudioEvent*		AddEvent(AudioEvent* newEvent);
    void            Dump(); // debug print
	
    fileBuffer*     GetAudio(QWORD readSample, DWORD samples = 0);
    
private:
	void			SetNewEnd(QWORD newEnd);
	void			SetNewStart(QWORD newStart);
	void			InsertBefore(AudioEvent* event);
	void			InsertAfter(AudioEvent* event);

public:
	AudioEvent*		Head() { return (prev) ? prev->Head() : this; }
	AudioEvent*		Tail() { return (next) ? next->Tail() : this; }

    AudioEvent*		EventAtSample(QWORD sampleLoc, AudioEvent* curEvent = NULL);
    AudioEvent*		EventBetweenSamples(QWORD sampleStart, QWORD sampleEnd, AudioEvent* curEvent = NULL);

    // a few support functions for recording operations
    void            Finish();
    bool            WriteData(QWORD samples);

};


class EDL 
{
public:
	EDL(hdLinkedList* files);
	~EDL();

private:
	AudioEvent*	    events;		// list of events

public:
	AudioEvent*	    EventAtSample(QWORD sampleLoc)                          { return (events) ? events->EventAtSample(sampleLoc) : NULL;}
    AudioEvent*     EventBetweenSamples(QWORD sampleStart, QWORD sampleEnd) { return (events) ? events->EventBetweenSamples(sampleStart, sampleEnd) : NULL;}
	hdFile*			FileAtSample(QWORD sampleLoc);
	void			Dump();

    QWORD           FirstSampleInEDL() { return (events) ? events->Head()->Start() : 0; }
    QWORD           LastSampleInEDL()  { return (events) ? events->Tail()->End()   : 0; }
};



#endif