// hdFile.h: interface for the hdFile class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_HDFILE_H__D7AC90BC_F48F_4A5A_942B_49B3B89EBAB2__INCLUDED_)
#define AFX_HDFILE_H__D7AC90BC_F48F_4A5A_942B_49B3B89EBAB2__INCLUDED_


#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "hdLinkedList.h"
#include "hdRiff.h"
#include <time.h>

#define SECTORBYTES  512

// used to store values, full struct used for Writes
//
struct BWF_FullHeader {
    RiffChunkHeader riff_header;
    DWORD           wave;
	BextFormatChunk	bextChunk;			// BWF chunk
	WaveFormatChunk waveChunk;			// WAVE chunk
	BaseIDChunk     idChunk;			// ID chunk (for id'ing two abutting max file size events)
    PadChunk        padChunk;           // padding for alignment purposes
	RiffChunkHeader pcmData;			// PCM data chunk
 };
 
 
class hdFile : public hdRiff, public hdListItem
{
public:
	hdFile(const char* filename, char, long, char, UINT64 loc);
	hdFile(const char* pathname, const char* filename);
	~hdFile();

    BWF_FullHeader  bwf;            

	QWORD           pcmDataOffset;		// offset of 'pcm_data' in wave file
	QWORD           audioDataOffset;	// offset of audio data in wave file
	QWORD			currentPosition;	// offset in current file read

	char*           fileName;			// name of file
	tm              fileCreated;		// date & time file was created

public:
	char*           GetCreationString();
	char*           GetFileSizeStr();
	
	// main member functions
	bool            Validate();
	bool            OpenForWrite();
	bool            OpenForPlay();
	bool            WriteData(char* fileBuf, QWORD bytes );
	bool            ReadAudioData(char* fileBuf, QWORD offset, QWORD bytes, QWORD& bytesRead);
	bool			SeekToSample(QWORD SampleIndex);
	bool			Close();
	bool            WriteHeader(bool newfile = false);
	bool			IsDualMono(hdFile* chkFile);
    bool			SameFileID(hdFile* compFile);

    QWORD    DataSize()              {return bwf.pcmData.ckSize;				     }
	DWORD    SampleRate()            {return bwf.waveChunk.data.nSamplesPerSec;      }
	WORD     BitsPerSample()         {return bwf.waveChunk.data.nBitsPerSample;      }
    WORD     BytesPerSample()        {return NumChannels()*(BitsPerSample()/8);  }
	WORD     NumChannels()           {return bwf.waveChunk.data.nChannels;           }
	void     GetCreateTime(tm&);
	char*    GetFileName()           {return fileName;                           }
	QWORD    NumSamples()            {return bwf.pcmData.ckSize / BytesPerSample();  }
	QWORD    GetStartSample()        {return bwf.bextChunk.data.GetStartLocation();  }
	QWORD    GetEndSample()          {return GetStartSample() + NumSamples();    }
    QWORD    SamplesToBytes(QWORD sp){return sp * BytesPerSample();				 }
    QWORD    BytesToSamples(QWORD bt){return bt / BytesPerSample();				 }

    	// our hdListItem overrides  (ugh)
	time_t    GetLocation()           { return mktime(&fileCreated);}
};


#endif // !defined(AFX_HDFILE_H__D7AC90BC_F48F_4A5A_942B_49B3B89EBAB2__INCLUDED_)
